local input = {}

input.bindings = {
	up = {
	  {
		device = "keyboard",
		key = "up"
	  },
	  {
		device = "keyboard",
		key = "w"
	  },
	  {
		device = "gamepad",
		type = "button",
		button = "dpup"
	  },
	  {
		device = "gamepad",
		type = "axis",
		axis = "lefty",
		direction = -1
	  }
	},
	down = {
	  {
		device = "keyboard",
		key = "down"
	  },
	  {
		device = "keyboard",
		key = "s"
	  },
	  {
		device = "gamepad",
		type = "button",
		button = "dpdown"
	  },
	  {
		device = "gamepad",
		type = "axis",
		axis = "lefty",
		direction = 1
	  }
	},
	left = {
	  {
		device = "keyboard",
		key = "left"
	  },
	  {
		device = "keyboard",
		key = "a"
	  },
	  {
		device = "gamepad",
		type = "button",
		button = "dpleft"
	  },
	  {
		device = "gamepad",
		type = "axis",
		axis = "leftx",
		direction = -1
	  }
	},
	right = {
	  {
		device = "keyboard",
		key = "right"
	  },
	  {
		device = "keyboard",
		key = "d"
	  },
	  {
		device = "gamepad",
		type = "button",
		button = "dpright"
	  },
	  {
		device = "gamepad",
		type = "axis",
		axis = "leftx",
		direction = 1
	  }
	},
	fire = {
	  {
		device = "keyboard",
		key = "j"
	  },
	  {
		device = "keyboard",
		key = "z"
	  },
	  {
		device = "gamepad",
		type = "button",
		button = "a"
	  }
	},
	special = {
	  {
		device = "keyboard",
		key = "k"
	  },
	  {
		device = "keyboard",
		key = "x"
	  },
	  {
		device = "gamepad",
		type = "button",
		button = "leftshoulder"
	  }
	},
	slow = {
	  {
		device = "keyboard",
		key = "space"
	  },
	  {
		device = "gamepad",
		type = "button",
		button = "rightshoulder"
	  }
	},
	bomb = {
	  {
		device = "keyboard",
		key = "l"
	  },
	  {
		device = "keyboard",
		key = "c"
	  },
	  {
		device = "gamepad",
		type = "button",
		button = "x"
	  }
	},
	pause = {
	  {
		device = "keyboard",
		key = "p"
	  },
	  {
		device = "gamepad",
		type = "button",
		button = "start"
	  }
	}
  }

input.active = {}
for k,v in pairs(input.bindings) do
	input.active[k] = false
end

input.joystick = love.joystick.getJoysticks()[1]
input.deadzone = 0.5

function input.update(dt)
	for k,v in pairs(input.bindings) do
		if input.isDown(k) then
			input.active[k] = true
		else
			if input.active[k] then
				onAction(k)
			end
			input.active[k] = false
		end
	end
end

function input.isDown(action)
	local bindings = input.bindings
	assert(bindings[action], "Action \"" .. action .. "\" not found in input system")
	for i,v in ipairs(bindings[action]) do
		if v.device == "keyboard" and love.keyboard.isDown(v.key) then
			return true
		end
		if v.device == "gamepad" and input.joystick and v.type == "button" and input.joystick:isGamepadDown(v.button) then
			return true
		end
		if v.device == "gamepad" and input.joystick and v.type == "hat" and string.find(input.joystick:getHat(1), v.direction) then
			return true
		end
		if v.device == "gamepad" and input.joystick and v.type == "axis" then
			local current_axis_value = input.joystick:getGamepadAxis(v.axis)
			if v.direction > 0 and current_axis_value > input.deadzone then
				return true
			elseif v.direction < 0 and current_axis_value < (input.deadzone * -1) then
				return true
			end
		end
	end
	return false
end

return input