Ball = Object:extend()


function Ball:new(x, y)
    self.sprite = love.graphics.newImage("assets/ball.png")
    self.radius = self.sprite:getWidth()/2

    self.cue = false
    
    self.restitution = 1
    self.friction = 0.5

    self.body = love.physics.newBody(world, x, y, "dynamic")
    self.shape = love.physics.newCircleShape(self.radius)
    self.fixture = love.physics.newFixture(self.body, self.shape)
    self.fixture:setRestitution(self.restitution)
    self.fixture:setFriction(self.friction)
    self.fixture:setUserData("Ball")

    self.idle = true
end

function Ball:tableFriction()
    local vx, vy = self.body:getLinearVelocity()
      
    local norm = math.sqrt(vx*vx + vy*vy)
    if norm > 0.01 then
        if ((vx or vy) > 300) or ((vx or vy) < -300) then
            self.body:applyForce(vx/norm*-120, vy/norm*-120)
        elseif ((vx or vy) > 200) or ((vx or vy) < -200) then
            self.body:applyForce(vx/norm*-105, vy/norm*-105)
        elseif ((vx or vy) > 150) or ((vx or vy) < -150) then
            self.body:applyForce(vx/norm*-95, vy/norm*-95)
        elseif ((vx or vy) > 90) or ((vx or vy) < -90) then
         self.body:applyForce(vx/norm*-70, vy/norm*-70)
        elseif ((vx or vy) > 50) or ((vx or vy) < -50) then
            self.body:applyForce(vx/norm*-60, vy/norm*-60)
        elseif ((vx or vy) > 5) or ((vx or vy) < -5) then
            self.body:applyForce(vx/norm*-55, vy/norm*-55)
        elseif ((vx or vy) > 1) or ((vx or vy) < -1) then
            self.body:applyForce(vx/norm*-45, vy/norm*-45)
        elseif ((vx or vy) > 0.1) or ((vx or vy) < -0.1) then
            self.body:applyForce(vx/norm*-40, vy/norm*-40)
        elseif ((vx or vy) > 0.001) or ((vx or vy) < -0.001) then
            self.body:setLinearVelocity(0, 0)
        end
    else
        self.body:setLinearVelocity(0, 0)
    end
    local omega = self.body:getAngularVelocity()
    local sign = omega < 0 and -1 or omega > 0 and 1 or 0
    self.body:applyTorque(sign*-100)
end

function Ball:setBallVelocity()
    self.vel_x, self.vel_y = self.body:getLinearVelocity()
    
end

function Ball:setBallPosition()
    self.x, self.y = self.body:getPosition()
end

function Ball:isBallIdle()
    if self.vel_x and self.vel_y ~= 0 then 
        self.idle = false
    else self.idle = true end
end

function Ball:drawBall()
    love.graphics.draw(self.sprite, self.x, self.y)
end

function Ball:update(dt)
    self:tableFriction()
    self:setBallVelocity()
    self:setBallPosition()
    self:isBallIdle()
    
end

function Ball:draw()
    self:drawBall()
end

return Ball