local push = require "libs/push"
CueBall = Ball:extend()


function CueBall:new(x, y)
    CueBall.super.new(self, x , y)
    self.cue = true

    self.shots = 2

    self.start_x = x
    self.start_y = y

    self.vec_x = 0
    self.vec_y = 0
    self.scope_rank = -220

    self.r_nx = nil
	self.r_ny = nil

    self.step = "idle"
    self.power = 0
    self.power_rate = 100
    self.power_dir = "up"
    self.canLaunch = true
    self.launchDelay = true

    self.fixture:setUserData("Cue Ball")
    self.sprite = love.graphics.newImage("assets/cueBall.png")
    self.radius = self.sprite:getWidth()/2
end

function CueBall:setCanLaunch(canLaunch)
    self.canLaunch = canLaunch
end

function CueBall:resetCueBall()
    self.body:setPosition(self.start_x, self.start_y)
    self.body:setLinearVelocity(0,0)
	self.step = "idle"
end

function CueBall:updateShots()
    self.shots = self.shots - 1
end

function CueBall:setPointerVector()
    local mouse_x, mouse_y = love.mouse.getPosition()
    mouse_x, mouse_y = push:toGame(mouse_x, mouse_y)
    local _x, _y = self.body:getPosition()

    if mouse_x and mouse_y ~= nil then
        self.vec_x = _x + self.radius - mouse_x
        self.vec_y = _y + self.radius - mouse_y
        self.vec_len = math.sqrt(self.vec_x^2 + self.vec_y^2)
        self.vec_x = - (self.vec_x / self.vec_len)
        self.vec_y = - (self.vec_y / self.vec_len)
        self.vec_angle = math.atan2((_y + self.radius) - mouse_y,(_x + self.radius) - mouse_x)
    end
end

function CueBall:drawCueBall()
    local _x, _y = self.body:getPosition()
    love.graphics.draw(self.sprite, _x, _y)
    love.graphics.setColor(1,0,0)
    love.graphics.circle("fill", _x + 6, _y + 6, 1)
    love.graphics.setColor(1,1,1)

end

function CueBall:launchBall(dt)
    function love.keypressed(key)
        if key == "space" and self.step == "idle" then
            if self.canLaunch then
                self.step = "prepare"
            end
        elseif key == "space" and self.step == "prepare" then
            self.step = "idle"
            self.body:applyLinearImpulse(self.vec_x * 5 * self.power, self.vec_y * 5 *self.power)
            self.power = 0
            self.step = "idle"
            self.power_dir = "up"
            self:updateShots()
          end
        end
        if self.step == "prepare" then
              if self.power_dir == "up" then
                self.power = math.floor(self.power + (dt * self.power_rate))
                
                if math.floor(self.power) >= 100 then
                  self.power_dir = "down"
                end
              end
              if  self.power_dir == "down" then
                self.power = (self.power - (dt * self.power_rate))
                
                if math.floor(self.power) <= 1 then
                  self.power_dir = "up"
                end
              end
        end
end

function CueBall:drawPowerBar()
    local _x, _y = self.body:getPosition()
    self.marker_x = (self.power * 0.77) + _x - 33

    CueBall.power_bar_sprite = love.graphics.newImage("assets/powerBar.png")
    CueBall.power_bar_marker_sprite = love.graphics.newImage("assets/powerBarMarker.png")

    love.graphics.push()
    love.graphics.draw(self.power_bar_sprite, _x - 35, _y + 20)
    love.graphics.draw(self.power_bar_marker_sprite, self.marker_x, _y + 22)
    love.graphics.pop()
end

function CueBall:drawPoolCue()
    local _x, _y = self.body:getPosition()
    self.cue_distance = (self.power * 0.4) + self.radius
    self.pool_cue_sprite = love.graphics.newImage("assets/poolCue.png")
    self.pool_cue_sprite:setFilter("nearest", "nearest")
    love.graphics.draw(self.pool_cue_sprite, _x + self.radius - (self.vec_x*self.cue_distance), _y + self.radius - (self.vec_y*self.cue_distance), self.vec_angle,1,1,0,9.5)
end

function CueBall:drawScopeLine()
    local _x, _y = self.body:getPosition()
    _x = _x + self.radius
    _y = _y + self.radius
    if self.r_nx then
        love.graphics.line(_x + (self.vec_x * (self.radius + 1)),  _y + (self.vec_y * (self.radius + 1)), self.r_nx + self.radius, self.r_ny + self.radius)
    else
        love.graphics.line(_x + (self.vec_x * (self.radius + 1)),  _y + (self.vec_y * (self.radius + 1)), _x  - (self.vec_x * self.scope_rank), _y  - (self.vec_y * self.scope_rank))
    end
end

function CueBall:update(dt)
    self:setBallPosition()
    self:setPointerVector()
    self:setBallVelocity()
    self:tableFriction()
    self:isBallIdle()
    self:launchBall(dt)

end

function CueBall:draw()
    self:drawCueBall()

    if self.step == "prepare" then
        self:drawPowerBar()
    end

    if self.canLaunch and self.idle then
        self:drawPoolCue()
        self:drawScopeLine()
    end

end

return CueBall