
local game = {}

function game:enter(old_enter)
	self.pool_table = PoolTable(BASE_WIDTH/2, BASE_HEIGHT/2)
	self.balls = {}
	self.ball_radius = 6
	self.cue_ball = CueBall(BASE_WIDTH/2 - 60, BASE_HEIGHT/2)
	self.cue_ball.shots = 10



	local x1 = self.pool_table.w/4
	local y = self.pool_table.h/2 - 14
	local rolls = 3
	local gapFactor = 1.02  -- 2%
  	local xform = love.math.newTransform(x1, 0,
     math.rad(-30),
     2*self.ball_radius*gapFactor, math.sqrt(3)*self.ball_radius*gapFactor,
     0, 0,
     0.5, 0)
	for yr = 0, 4 do
		for xr = 0, 4 do
		if xr + yr < rolls then -- within the triangle
			x1, y = xform:transformPoint(xr, yr)
			table.insert(self.balls, Ball(BASE_WIDTH/2 + x1 - 20, BASE_HEIGHT/2 + y))
		end
		end
	end
end

function game:isAllBallsIdle()
	local count = 0
	
	for i, ball in ipairs(self.balls) do
		if ball.idle == true then
			count = count + 1
		end
	end
	if count == #self.balls then
		self.cue_ball.canLaunch = true
		return true
	else
		self.cue_ball.canLaunch = false
		return false
	end
end

function game:testScopeTrajectory(target)
	function table.min(t)
		local k, v = next(t)
		for k2, v2 in pairs(t) do
		  if v2 < v then
			k, v = k2, v2
		  end
		end
		return k, v
	  end
	local fracs = {}
	local min_frac = 0
	local _x, _y = self.cue_ball.body:getPosition()
	local x1 = _x + (self.cue_ball.vec_x * (self.cue_ball.radius + 1))
	local y1 = _y + (self.cue_ball.vec_y * (self.cue_ball.radius + 1))
	local x2 = _x  - (self.cue_ball.vec_x * self.cue_ball.scope_rank)
	local y2 = _y  - (self.cue_ball.vec_y * self.cue_ball.scope_rank)
	local max_fraction = 1
	for i, ball in ipairs(self.balls) do
		local xn, yn, fraction = ball.fixture:rayCast(x1, y1, x2, y2, max_fraction, _x, _y, 10)
		if fraction then
			fracs[i] = fraction
		else
			fracs[i] = 2
		end
	end
	local k, v = table.min(fracs)
	min_frac = v
	if min_frac < 2 then
		self.cue_ball.r_nx = x1 + (x2 - x1) * min_frac
		self.cue_ball.r_ny = y1 + (y2 - y1) * min_frac
	else
		self.cue_ball.r_nx = nil
		self.cue_ball.r_ny = nil
	end
end



function game:update(dt)
	self:isAllBallsIdle()
	self:testScopeTrajectory()
	self.cue_ball:update(dt)
	for i, ball in ipairs(self.balls) do
		ball:update(dt)
		for k, hole in ipairs(self.pool_table.holes) do
			if hole:detectCollision(ball) then
				if not ball.cue then
					table.remove(self.balls, i)
					ball.body:destroy()
				else
					ball:resetCueBall()
				end
			end
		end
	end
end

function game:draw()
	self.pool_table:draw()
	self.cue_ball:draw()
	for k, v in ipairs(self.balls) do
		v:draw()
	end

	love.graphics.setColor(1, 1, 1)
	love.graphics.print("tacadas restantes: " .. self.cue_ball.shots, 20, 20)

	if #self.balls == 1 and self:isAllBallsIdle() then
		love.graphics.print("Você venceu", BASE_WIDTH/2 - 50, BASE_HEIGHT/2)
	end

	if self.cue_ball.shots <= 0 and self:isAllBallsIdle() then
		love.graphics.print("Você perdeu", BASE_WIDTH/2 - 50, BASE_HEIGHT/2)
	end
end

return game