Object = require "libs.classic"

StateManager = require "libs.hump.gamestate"
Timer = require "libs.hump.timer"
Signals = require "libs.hump.signal"

Vector = require "libs.brinevector"
Vector.unpack = function(v)
	return v.x, v.y
end
Vector.unpackFloor = function(v)
	return math.floor(v.x), math.floor(v.y)
end
Vector.unpackRound = function(v)
	return lume.round(v.x), lume.round(v.y)
end

util = require "framework.util"
input = require "framework.input"
local push = require "libs/push"

Ball = require "game.ball"
CueBall = require "game.cueBall"
Hole = require "game.hole"
PoolTable = require "game.poolTable"

Menu = require "framework.menu"

states = {
	title = require "game.states.title",
  game = require "game.states.game"
}

global_inputs = {
  up = {
    {
      device = "keyboard",
      key = "up"
    },
    {
      device = "keyboard",
      key = "w"
    },
    {
      device = "gamepad",
      type = "button",
      button = "dpup"
    },
    {
      device = "gamepad",
      type = "axis",
      axis = "lefty",
      direction = -1
    }
  },
  down = {
    {
      device = "keyboard",
      key = "down"
    },
    {
      device = "keyboard",
      key = "s"
    },
    {
      device = "gamepad",
      type = "button",
      button = "dpdown"
    },
    {
      device = "gamepad",
      type = "axis",
      axis = "lefty",
      direction = 1
    }
  },
  left = {
    {
      device = "keyboard",
      key = "left"
    },
    {
      device = "keyboard",
      key = "a"
    },
    {
      device = "gamepad",
      type = "button",
      button = "dpleft"
    },
    {
      device = "gamepad",
      type = "axis",
      axis = "leftx",
      direction = -1
    }
  },
  right = {
    {
      device = "keyboard",
      key = "right"
    },
    {
      device = "keyboard",
      key = "d"
    },
    {
      device = "gamepad",
      type = "button",
      button = "dpright"
    },
    {
      device = "gamepad",
      type = "axis",
      axis = "leftx",
      direction = 1
    }
  },
  fire = {
    {
      device = "keyboard",
      key = "j"
    },
    {
      device = "keyboard",
      key = "z"
    },
    {
      device = "gamepad",
      type = "button",
      button = "a"
    }
  },
  special = {
    {
      device = "keyboard",
      key = "k"
    },
    {
      device = "keyboard",
      key = "x"
    },
    {
      device = "gamepad",
      type = "button",
      button = "leftshoulder"
    }
  },
  slow = {
    {
      device = "keyboard",
      key = "space"
    },
    {
      device = "gamepad",
      type = "button",
      button = "rightshoulder"
    }
  },
  bomb = {
    {
      device = "keyboard",
      key = "l"
    },
    {
      device = "keyboard",
      key = "c"
    },
    {
      device = "gamepad",
      type = "button",
      button = "x"
    }
  },
  pause = {
    {
      device = "keyboard",
      key = "p"
    },
    {
      device = "gamepad",
      type = "button",
      button = "start"
    }
  }
}

love.window.setMode(1280,720)


world = love.physics.newWorld(0, 0, true)
love.physics.setMeter(10)
love.graphics.setDefaultFilter("nearest", "nearest")
love.graphics.setLineStyle("rough")

local windowWidth, windowHeight = love.graphics.getDimensions()
windowWidth, windowHeight = windowWidth, windowHeight --make the window a bit smaller than the screen itself

push:setupScreen(BASE_WIDTH, BASE_HEIGHT, windowWidth, windowHeight, {
  fullscreen = false,
  pixelperfect = true,
  canvas = true,
  
  })

function love.load()
  BASE_CANVAS = love.graphics.newCanvas(BASE_WIDTH, BASE_HEIGHT)
  StateManager.switch(states.game)
end

function love.update(dt)
  world:update(dt)
  StateManager.update(dt)
  input.update(dt)

end

function love.draw()
	push:apply("start")
  StateManager.draw()
	love.graphics.setColor(1, 1, 1)
	love.graphics.print(tostring(love.timer.getFPS( )) .. "fps", 3, 0)
  push:apply("end")
  
end

function onAction(action)
	StateManager.onInputAction(action)
end

function love.keyreleased(key)
	StateManager.keyreleased(key)

end